import pickle as pkl
import zlib

import diskcache
from diskcache import UNKNOWN


class CompressedDisk(diskcache.Disk):
    def __init__(self, directory, compress_level=5, **kwargs):
        self.compress_level = compress_level
        super(CompressedDisk, self).__init__(directory, **kwargs)

    def get(self, key, raw):
        data = super(CompressedDisk, self).get(key, raw)
        return pkl.loads(zlib.decompress(data))

    def store(self, value, read, key=UNKNOWN):
        if not read:
            value = zlib.compress(pkl.dumps(value, protocol=pkl.HIGHEST_PROTOCOL), self.compress_level)
        return super(CompressedDisk, self).store(value, read, key=key)

    def fetch(self, mode, filename, value, read):
        data = super(CompressedDisk, self).fetch(mode, filename, value, read)
        if not read:
            data = pkl.loads(zlib.decompress(data))
        return data
