import dask.dataframe as dd
import pandas as pd
from dask.diagnostics import ProgressBar
from distributed import Client, LocalCluster

src_file = '/ccollins/backup/aggregated_data2.h5'
sweep_variables = ['Release_number', 'Copy_To_Likelihood', 'ITN_Coverage', 'Transmission_To_Human', 'NodeID', 'Time']
channels = ['Prevalence', 'Daily_EIR', 'Adult_Vectors', 'New_Clinical_Cases']


# Here we use Dask to loop through data-sets in our HDF5 file that we used to aggregate data from step 1


def run_calc():
    final = dict()
    for aggregator in ['SpatialAggregator', 'GMEstablishmentAggregator']:
        df = dd.read_hdf(src_file, aggregator)
        print('Grouping')
        with ProgressBar():
            out = df.groupby(sweep_variables)[channels].mean().compute()
        print('Compute')
        final[aggregator] = out

    # merge the group bys for final data form
    final_ds = pd.merge(final['SpatialAggregator'], final['GMEstablishmentAggregator'], how='left', on=sweep_variables)
    final_ds.to_csv('final.csv', index=False)


if __name__ == '__main__':
    cluster = LocalCluster()
    # setup our client
    client = Client(cluster, processes=False)  # ,  memory_limit='8GB')
    run_calc()
