import dask.dataframe as dd
import pandas as pd
from dask.diagnostics import ProgressBar
from distributed import Client, LocalCluster

src_file = 'aggregated_data_debu.h5'
sweep_variables = ['Release_Number', 'Copy_To_Likelihood', 'ITN_Coverage', 'Transmission_To_Human', 'node', 'time']
channels = ['Prevalence', 'Daily_EIR', 'Adult_Vectors', 'New_Clinical_Cases']

# Here we use Dask to loop through data-sets in our HDF5 file that we used to aggregate data from step 1
pd.options.mode.chained_assignment = None

def run_calc(client):
    final = dict()
    for aggregator in ['SpatialAnalyzer', 'GMEstablishmentAnalyzer']:
        df = dd.read_hdf(src_file, aggregator, mode='r')
        print('Grouping')
        with ProgressBar():
            df = df.groupby(sweep_variables)[channels].mean().compute()
            del df
        print('Compute')
        final[aggregator] = out

    # merge the group bys for final data form
    final_ds = pd.merge(final['SpatialAggregator'], final['GMEstablishmentAggregator'], how='left', on=sweep_variables)
    final_ds.to_csv('final.csv', index=False)


if __name__ == '__main__':
    cluster = LocalCluster()
    # setup our client
    client = Client(cluster, memory_limit='6GB', processes=False)
    #client = Client('127.0.0.1:8786')
    run_calc(client)
